/*
 * The POG project
 * Copyright 2000 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */


#include <math.h>

#define WANT_SINCOS
#include "porting.h" // need for sincos

#include "mygl.h"
#include "3DMath.h"




void MatrixTranspose(GLMatrix *MD, GLMatrix *MS)
/* source a destination matrix must be not same */
{
  assert(MS != MD);

  (*MD)[0][0] = (*MS)[0][0];  (*MD)[1][0] = (*MS)[0][1];  (*MD)[2][0] = (*MS)[0][2];  (*MD)[3][0] = (*MS)[0][3];
  (*MD)[0][1] = (*MS)[1][0];  (*MD)[1][1] = (*MS)[1][1];  (*MD)[2][1] = (*MS)[1][2];  (*MD)[3][1] = (*MS)[1][3];
  (*MD)[0][2] = (*MS)[2][0];  (*MD)[1][2] = (*MS)[2][1];  (*MD)[2][2] = (*MS)[2][2];  (*MD)[3][2] = (*MS)[2][3];
  (*MD)[0][3] = (*MS)[3][0];  (*MD)[1][3] = (*MS)[3][1];  (*MD)[2][3] = (*MS)[3][2];  (*MD)[3][3] = (*MS)[3][3];
}


void MatrixLoadIdentity(GLMatrix *M)
{
  memset(M, 0, sizeof(GLMatrix));
  (*M)[0][0] = 1;
  (*M)[1][1] = 1;
  (*M)[2][2] = 1;
  (*M)[3][3] = 1;
}


void MatrixMoveForward(GLMatrix (*M), GLfloat S)
{
  (*M)[3][0] += (*M)[2][0] * S;
  (*M)[3][1] += (*M)[2][1] * S;
  (*M)[3][2] += (*M)[2][2] * S;
}


void MatrixMoveBackward(GLMatrix (*M), GLfloat S)
{
  (*M)[3][0] -= (*M)[2][0] * S;
  (*M)[3][1] -= (*M)[2][1] * S;
  (*M)[3][2] -= (*M)[2][2] * S;
}


void MatrixMove(GLMatrix (*M), float Forward, float LR, float DU)
{
  (*M)[3][0] +=  (*M)[2][0] * Forward  +  (*M)[0][0] * LR  + (*M)[1][0] * DU;
  (*M)[3][1] +=  (*M)[2][1] * Forward  +  (*M)[0][1] * LR  + (*M)[1][1] * DU;
  (*M)[3][2] +=  (*M)[2][2] * Forward  +  (*M)[0][2] * LR  + (*M)[1][2] * DU;
}


void MatrixRotate(GLMatrix *M, const GLfloat RotLR, const GLfloat RotDU, const GLfloat Rol)
{
  GLfloat CosLR,SinLR,CosUD,SinUD,CosOl,SinOl;
  GLMatrix N1,N2;
  double K1,K2;

  /* sins a cosins of rotation */
  sincos(-RotLR, &K1, &K2);  SinLR=K1; CosLR=K2;
  sincos( RotDU, &K1, &K2);  SinUD=K1; CosUD=K2;
  sincos( Rol,   &K1, &K2);  SinOl=K1; CosOl=K2;

  /* copy of original matrix */
  memcpy(&N2, M, sizeof(GLMatrix));

  /* some mathematical magic */
  N1[0][0] =  CosLR*CosOl;
  N1[0][1] = -CosLR*SinOl;
  N1[0][2] =  SinLR;
  N1[1][0] =  SinUD*SinLR*CosOl+CosUD*SinOl;
  N1[1][1] = -SinUD*SinLR*SinOl+CosUD*CosOl;
  N1[1][2] = -SinUD*CosLR;
  N1[2][0] = -CosUD*SinLR*CosOl+SinUD*SinOl;
  N1[2][1] =  CosUD*SinLR*SinOl+SinUD*CosOl;
  N1[2][2] =  CosUD*CosLR;

  /* matrix multiplication */
  (*M)[0][0] = N2[0][0]*N1[0][0] + N2[1][0]*N1[1][0] + N2[2][0]*N1[2][0];
  (*M)[1][0] = N2[0][0]*N1[0][1] + N2[1][0]*N1[1][1] + N2[2][0]*N1[2][1];
  (*M)[2][0] = N2[0][0]*N1[0][2] + N2[1][0]*N1[1][2] + N2[2][0]*N1[2][2];
  (*M)[0][1] = N2[0][1]*N1[0][0] + N2[1][1]*N1[1][0] + N2[2][1]*N1[2][0];
  (*M)[1][1] = N2[0][1]*N1[0][1] + N2[1][1]*N1[1][1] + N2[2][1]*N1[2][1];
  (*M)[2][1] = N2[0][1]*N1[0][2] + N2[1][1]*N1[1][2] + N2[2][1]*N1[2][2];
  (*M)[0][2] = N2[0][2]*N1[0][0] + N2[1][2]*N1[1][0] + N2[2][2]*N1[2][0];
  (*M)[1][2] = N2[0][2]*N1[0][1] + N2[1][2]*N1[1][1] + N2[2][2]*N1[2][1];
  (*M)[2][2] = N2[0][2]*N1[0][2] + N2[1][2]*N1[1][2] + N2[2][2]*N1[2][2];
}


void MatrixNormalize(GLMatrix (*M))
{
  #define sqr(x) ((x)*(x))
  GLfloat S;
  S = (float)1.0 / sqrt(sqr((*M)[0][0]) + sqr((*M)[0][1]) + sqr((*M)[0][2]));
  (*M)[0][0] *= S;
  (*M)[0][1] *= S;
  (*M)[0][2] *= S;

  S = (float)1.0 / sqrt(sqr((*M)[1][0]) + sqr((*M)[1][1]) + sqr((*M)[1][2]));
  (*M)[1][0] *= S;
  (*M)[1][1] *= S;
  (*M)[1][2] *= S;

  S = (float)1.0 / sqrt(sqr((*M)[2][0]) + sqr((*M)[2][1]) + sqr((*M)[2][2]));
  (*M)[2][0] *= S;
  (*M)[2][1] *= S;
  (*M)[2][2] *= S;
}



