/*
 * The POG project
 * Copyright 2001 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */

#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <iostream.h>
#include <list.h>
#include <new.h>
#pragma hdrstop // for C++ Builder compiler speed up

#define WANT_NOTHROW
#define WANT_SQR
#include "porting.h"
#include "ArtificialIntelligence.h"
#include "Scene.h"
#include "Log.h"



/*
class NeuralNetwork
{
protected:
	InputLayer *inputlayer;
	HidenLayer *hidenlayers;
	OutputLayer *outputlayer;
	int layerscount;
public:
	void setInput(int I, float V)  { ((InputNeuron*)(inputlayer->getNeuron(I)))->Input(V); }
	float getOutput(int I)  { return outputlayer->getNeuron(I)->Output(); }

	Layer* getLayer(int I);
	inline int LayersCount()  { return layerscount; }

	inline Neuron* getNeuron(int L, int I)  { return getLayer(L)->getNeuron(I); }

	int Initialize(int LayersCount);
	void Update()  { for(int i=0; i<layerscount-1; i++) hidenlayers[i].Update(); outputlayer->Update(); }

	NeuralNetwork()  { inputlayer = NULL; hidenlayers = NULL; outputlayer = NULL; layerscount = 0; }
	virtual ~NeuralNetwork()  { delete inputlayer; delete[] hidenlayers; delete outputlayer; };

	void Save(ostream &S);
	int Load(istream &S);
};








Layer* NeuralNetwork::getLayer(int I)
{
	assert((I>=0) && (I<layerscount));
	if(I==0) return (Layer*)inputlayer;
	else if(I==layerscount-1) return (Layer*)outputlayer;
		else return &((Layer*)hidenlayers)[I-1];
};


void ActiveNeuron::Save(ostream &S)
{
	S << "ActiveNeuron" << inputscount;
	for(int i=0; i<inputscount; i++)
	{
		S << inputs[i].getInputNeuron()->getLayer()->getIndex();
		S << inputs[i].getInputNeuron()->getIndex();
		S << inputs[i].getWeight();
	}
};


int ActiveNeuron::Load(istream &S)
{
	char t[13]; S >> t;
	if(strcmp(t,"ActiveNeuron") != 0) return 0;
	int count; S >> count;
	if (!Initialize(count-2)) return 0;
	for(int i=0; i<inputscount; i++)
	{
		int li; S >> li;
		int ni; S >> ni;
		NeuralNetwork *nn = layer->getNetwork();
		float w; S >> w;
		inputs[i].Initialize(nn->getNeuron(li, ni), w);
	}
	return 1;
};


int NeuralNetwork::Initialize(int LayersCount)
{
	assert(LayersCount>=2);
	delete inputlayer; delete[] hidenlayers; delete outputlayer;
	inputlayer = new(nothrow) InputLayer;
	hidenlayers = new(nothrow) HidenLayer[LayersCount-2];
	outputlayer = new(nothrow) OutputLayer;
	if((inputlayer==NULL) || (outputlayer==NULL) ||
		((hidenlayers==NULL) && (layerscount!=2))) return 0;
	layerscount=LayersCount;
	return 1;
};


void NeuralNetwork::Save(ostream &S)
{
	S << "NeuralNetwork";
	S << layerscount;
	inputlayer->Save(S);
	for(int i=0; i<layerscount-2; i++) hidenlayers[i].Save(S);
	outputlayer->Save(S);

	for(int j=0; j<layerscount-2; j++)
		for(int i=0; i<hidenlayers[j].neuronscount; i++)
			hidenlayers[j].neurons[i].Save(S);
};


int NeuralNetwork::Load(istream &S)
{
	char t[14]; S >> t;
	if(strcmp(t,"NeuralNetwork") != 0) return 0;
	int count; S >> count;
	if (!Initialize(count-2)) return 0;

	if(!inputlayer->Load(S)) return 0;
	for(int i=0; i<layerscount; i++)
		if(!hidenlayers[i].Load(S)) return 0;
	if(!outputlayer->Load(S)) return 0;

	for(int j=0; j<layerscount-2; j++)
		for(int i=0; i<hidenlayers[j].neuronscount; i++)
			if(!hidenlayers[j].neurons[i].Load(S)) return 0;
	for(int i=0; i<outputlayer->neuronscount; i++)
		if(!outputlayer->neurons[i].Load(S)) return 0;

	return 1;
};*/



int BPNNetwork::Init(int LayersMax)
{
	delete[] layers;
	layerscount=0;
	delete[] Inputs;
	InputsCount=0;

	assert(LayersMax>=2);

	layers = new(nothrow) Layer[LayersMax-1];
	if (layers!=NULL)
	{
		layersmax = LayersMax;
		return 1;
	}
	else
	{
		layersmax = 0;
		return LayersMax!=1;
	}
}


int BPNNetwork::AddLayer(int NeuronsCount)
{
	assert(layerscount<layersmax);

	if (layerscount!=0)
	{
		if (layers[layerscount-1].Init(NeuronsCount))
		{
			layerscount++;
			return 1;
		}
		else
			return 0;
	}
	else
	{
		Inputs = new(nothrow) float[NeuronsCount];
		if (Inputs!=NULL)
		{
			InputsCount = NeuronsCount;
			layerscount++;
			return 1;
		}
		else
		{
			if (NeuronsCount==0)
			{
				layerscount++;
				return 1;
			}
			else
				return 0;
		}
	}
}


void BPNNetwork::Update()
{
	assert(layerscount >= 2);

	Layer *L;
	Neuron  *N;
	float suma;

	L = &layers[0];
	for (int j=0; j<L->neuronscount; j++)
	{
		N = &L->neurons[j];
		suma = 0;
		for (int k=0; k<InputsCount; k++)
		{
			suma += N->weights[k] * Inputs[k];
		}
		N->output = 1.0/(1.0+exp(-suma));
	}

	Layer *PL;
	for (int i=1; i<layerscount-1; i++)
	{
		L = &layers[i];
		PL = &layers[i-1];
		for (int j=0; j<L->neuronscount; j++)
		{
			N = &L->neurons[j];
			suma = 0;
			for (int k=0; k<PL->neuronscount; k++)
			{
				suma += N->weights[k]*PL->neurons[k].output;
			}
			N->output = 1.0/(1.0+exp(-suma));
		}
	}
}


void BPNNetwork::PropagateError(float Eta)
{
	assert(layerscount >= 2);

	// set deltas to zero
	Layer *L;
	for (int i=0; i<layerscount-2; i++)
	{
		L = &layers[i];
		for (int j=0; j<L->neuronscount; j++)
			L->neurons[j].delta = 0;
	}

	// propagate deltas
	Layer *PL;
	Neuron *N;
	for (int i=layerscount-2; i>=1; i--)
	{
		L = &layers[i];
		PL = &layers[i-1];
		for (int j=0; j<L->neuronscount; j++)
		{
			N = &L->neurons[j];
			for (int k=0; k<PL->neuronscount; k++)
			{
				PL->neurons[k].delta += N->delta * N->weights[k];
			}
		}
		for (int j=0; j<PL->neuronscount; j++)
		{
			N = &PL->neurons[j];
			N->delta = N->output * (1 - N->output) * N->delta;
		}
	}

	// new weights
	L = &layers[0];
	for (int j=0; j<L->neuronscount; j++)
	{
		N = &L->neurons[j];
		for (int k=0; k<InputsCount; k++)
		{
			N->weights[k] += Eta * N->delta * Inputs[k];
		}
	}

	for (int i=1; i<layerscount-1; i++)
	{
		L = &layers[i];
		PL = &layers[i-1];
		for (int j=0; j<L->neuronscount; j++)
		{
			N = &L->neurons[j];
			for (int k=0; k<PL->neuronscount; k++)
			{
				N->weights[k] += Eta * N->delta * PL->neurons[k].output;
			}
		}
	}
}



#define WATCHED_OBJECTS 4
#define INPUTS_PER_OBJECT 5
#define OUTPUTS 4


bool operator <(const PStatic &a, const PStatic &b) {
	return a.dist < b.dist;
}


void RocketDriver::TimeTick(double DTime)
{
	PStatic tmp;
	distancelist.clear();

	scMobile *M = MobileList.First;
	while (M != NULL)
	{
		if((M!=attachedrocket) && (M!=ThisPlayer))
		{
			tmp.P = M;
			tmp.dx = M->GetPosX() - attachedrocket->GetPosX();
			tmp.dy = M->GetPosY() - attachedrocket->GetPosY();
			tmp.dz = M->GetPosZ() - attachedrocket->GetPosZ();
			tmp.dist = sqrt(sqr(tmp.dx) + sqr(tmp.dy) + sqr(tmp.dz));
			tmp.enemy = 1.0;
			distancelist.push_back(tmp);
		}
		M = (scMobile*)M->Next;
	}
	scStatic *S = StaticList.First;
	while (S != NULL)
	{
		tmp.P = S;
		tmp.dx = S->GetPosX() - attachedrocket->GetPosX();
		tmp.dy = S->GetPosY() - attachedrocket->GetPosY();
		tmp.dz = S->GetPosZ() - attachedrocket->GetPosZ();
		tmp.dist = sqrt(sqr(tmp.dx) + sqr(tmp.dy) + sqr(tmp.dz));
		tmp.enemy = 0.0;
		distancelist.push_back(tmp);
		S = (scStatic*)S->Next;
	}

	distancelist.sort();

	// send data to neural network
	list<PStatic>::iterator listit = distancelist.begin();
	int n = -1;
	for(int i=0; i<WATCHED_OBJECTS; i++)
	{
		network->Inputs[++n] = listit->dx;
		network->Inputs[++n] = listit->dy;
		network->Inputs[++n] = listit->dz;
		network->Inputs[++n] = listit->enemy;
		listit++;
	}

	// update neural network
	network->Update();

	// receive data from neural network
	GLfloat tx = network->getOutput(0);
	GLfloat ty = network->getOutput(1);
	GLfloat tz = network->getOutput(2);
#define Clip(X) { if ((X)>20.0) (X) = 20.0; else if ((X)<-20.0) (X) = -20.0; }
	Clip(tx);
	Clip(ty);
	Clip(tz);
#undef Clip(X)
	attachedrocket->SetSpeed(tx,ty,tz);
};


int TrainingArena::BuildNet(BPNNetwork *nn)
{
	logInfo(modAI, "Building new net.");

	int LC = 4;
	int NC[] = { WATCHED_OBJECTS*INPUTS_PER_OBJECT, WATCHED_OBJECTS*INPUTS_PER_OBJECT,
			WATCHED_OBJECTS*INPUTS_PER_OBJECT, OUTPUTS };

	if(!nn->Init(LC)) return 0;
	Layer *l;
	Neuron *n;

	// input layer
	if(!nn->AddLayer(NC[0])) return 0;

	// hiden and output layers
	for(int i=1; i<LC; i++)
	{
		l = &nn->layers[i-1];
		if(!nn->AddLayer(NC[i])) return 0;
		for(int j=0; j<l->neuronscount; j++)
		{
			n = &l->neurons[j];
			if(!n->Init(NC[i-1])) return 0;
			for(int k=0; k<n->weightscount; k++)
				n->weights[k] = 0.3 - random(600)/1000;
		}
	}

	return 1;
};


struct {
	float input[INPUTS_PER_OBJECT*WATCHED_OBJECTS];
	float output[OUTPUTS];
} TrainingSet[] =
{
	// fire
	{
		{
			 40,  0,  0, 40,  1,
			-30,-20,-20, 41,  1,
			-30, 20,-20, 41,  1,
			-30,  0, 20, 36,  1
		},{
			0.5, 0.5, 0.5, 1
		}
	},{
		{
			 40,  0,  0, 40,  1,
			-30,-20, 20, 41,  1,
			-30, 20, 20, 41,  1,
			-30,  0,-20, 36,  1
		},{
			0.5, 0.5, 0.5, 1
		}
	},
	// aim right
	{
		{
			 40, 20,  0, 40,  1,
			-30,-20,-20, 41,  1,
			-30, 20,-20, 41,  1,
			-30,  0, 20, 36,  1
		},{
			0.5, 1.0, 0.5, 0
		}
	},
	// aim left
	{
		{
			 40,-20,  0, 40,  1,
			-30,-20, 20, 41,  1,
			-30, 20, 20, 41,  1,
			-30,  0,-20, 36,  1
		},{
			0.5, 0.0, 0.5, 0
		}
	},
	// aim up
	{
		{
			 40,  0, 20, 40,  1,
			-30,-20,-20, 41,  1,
			-30, 20,-20, 41,  1,
			-30,  0, 20, 36,  1
		},{
			0.5, 0.5, 1.0, 0
		}
	},
	// aim down
	{
		{
			 40,  0,-20, 40,  1,
			-30,-20, 20, 41,  1,
			-30, 20, 20, 41,  1,
			-30,  0,-20, 36,  1
		},{
			0.5, 0.5, 0.0, 0
		}
	},
};


void TrainingArena::TeachBasic(BPNNetwork *nn)
{
	logInfo(modAI, "Teaching net.");

	for(int t=0; t<10001; t++)
	{
		if (t%1000 == 0) logLevelTable[modAI] = logLevelExInfos;
		else logLevelTable[modAI] = logLevelWarnings;

		float GlobalError = 0;
		float CurrentError;

		for(int r=0; r<6; r++)
		{
			int i;

			if (t%100 == 0) i = r;
			else i = random(6);

			logExInfo(modAI, "Teaching %i set:", i);
			CurrentError = 0;

			// set inputs
			for(int j=0; j<WATCHED_OBJECTS*INPUTS_PER_OBJECT; j++)
			{
				nn->Inputs[j] = TrainingSet[i].input[j];
			}

			// update
			nn->Update();

			// correct outputs
			Layer *L;
			Neuron *N;
			L = &nn->layers[nn->layerscount-2];
			for(int j=0; j<L->neuronscount; j++)
			{
				N = &L->neurons[j];
				nn->CorrectOutput(j, TrainingSet[i].output[j]);
				float E = nn->getOutput(j) - TrainingSet[i].output[j];
				CurrentError += E*E;
				logExInfo(modAI, "Output %2i, right value: %5f, current value %5f, delta: %5f.", j, TrainingSet[i].output[j], nn->getOutput(j), N->delta);
			}

			// propagate errors
			nn->PropagateError(5);

			CurrentError /= 2;

			logExInfo(modAI, "Set %i finished with error %f.", i, CurrentError);
		}
		GlobalError += CurrentError;
		logInfo(modAI, "AI loop %i, error: %f.\n", t, GlobalError);
	}
};


TrainingArena AIArena;

int AITest(void)
{
//	BPNNetwork AINet;
//	AIArena.BuildNet(&AINet);
//	AIArena.TeachBasic(&AINet);
//	return 0;
	return 1;
};
