/*
 * The POG project
 * Copyright 2001 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */

#ifndef _ArtificalInteligence_h_
#define _ArtificalInteligence_h_

#include <list.h>
#define WANT_NEW
#include "porting.h"
#include "SceneClasses.h"


class Neuron
{
public:
	float output;
	float *weights;
	int weightscount;
	float delta;

	Neuron() : weights(NULL), weightscount(0) {}
	~Neuron()  { delete[] weights; }

	int Init(int WeightsCount)  { delete[] weights; weights = new(nothrow) float[WeightsCount];
			if (weights!=NULL) { weightscount = WeightsCount; return 1; }
			else { weightscount = 0; return WeightsCount==0; } };
};


class Layer
{
public:
	Neuron* neurons;
	int neuronscount;

	Layer() : neurons(NULL), neuronscount(0) {}
	~Layer()  { delete[] neurons; }

	int Init(int NeuronsCount)  { delete[] neurons; neurons = new(nothrow) Neuron[NeuronsCount];
			if (neurons!=NULL) { neuronscount = NeuronsCount; return 1; }
			else { neuronscount = 0; return NeuronsCount==0; } };
};


class BPNNetwork
{
protected:
	int layersmax;
public:
	Layer *layers;
	int layerscount;
	float *Inputs;
	int InputsCount;

	float getOutput(int I)  { assert(layerscount>=2); assert(I<layers[layerscount-2].neuronscount);
			return layers[layerscount-2].neurons[I].output; }
	void CorrectOutput(int I, float V)  { assert(layerscount>=2); assert(I<layers[layerscount-2].neuronscount);
			Neuron *N = &layers[layerscount-2].neurons[I];
			N->delta = N->output * (1 - N->output) * (V - N->output); }

	void Update();
	void PropagateError(float Eta);

	BPNNetwork() : Inputs(NULL), InputsCount(0), layers(NULL), layerscount(0) {}
	~BPNNetwork()  { delete[] Inputs; delete[] layers; }

	int Init(int LayersMax);
	int AddLayer(int NeuronsCount);
};




typedef struct {
	scStatic *P;
	float dx;
	float dy;
	float dz;
	float dist;
	float enemy;
} PStatic;


class RocketDriver
{
private:
	scRocket *attachedrocket;
	BPNNetwork *network;
	list<PStatic> distancelist;
public:
	void TimeTick(double DTime);

//	int LoadProfile(ostream &S)  { return network->Load(S); };
	int LoadProfile(char* FileName);

	RocketDriver()  { attachedrocket = NULL; }
	RocketDriver(scRocket *AttachedRocket)  { attachedrocket = AttachedRocket; }
	~RocketDriver();
	void AttachRocket(scRocket *AttachedRocket)  { attachedrocket = AttachedRocket; }
};


class TrainingArena
{
private:
public:
	int BuildNet(BPNNetwork *nn);
	void TeachBasic(BPNNetwork *nn);

};

//extern TrainingArena AIArena;
int AITest(void);

#endif /* _ArtificalInteligence_h_ */

