/* Modul CockpitMap - vykresleni mrizkove mapy v kokpitu
   Vytvoril Jaroslav Kadlec
   Datum    20.3.2001
*/

#ifdef WINDOZE
#include <windows.h>
#endif
#include <math.h>
#include <list>
#include "mygl.h"
#pragma hdrstop // for C++ Builder compiler speed up

#include "gui.h"
#include "CockpitMap.h"
#include "Scene.h"


int MapXpos = 540;                                //pozice mapy
int MapYpos = 430;
int MapSize =  70;                                //velikost mapy
int CMRange =  70;                                //zobrazeny prostor v mape

float MapSlope = 0.5;                             //% sklon mapy (50%)
float CMRdelta = ((float) MapSize) / CMRange;     //delta pro hash do mapy

float CockpitMapFrnt = MapSize+10;                //1/2 celni strany
float CockpitMapRear = MapSize-10;                //1/2 zadni strany

float deltaFy = (CockpitMapFrnt/MapSize - 1)/MapSize;   //hash konst celni casti mapy
float deltaRy = (CockpitMapRear/MapSize - 1)/MapSize;   //hash konst zadni casti mapy

bool CMTransparent = 0;                     //pruhlednost mapy
int DLCockpitMap;                           //Nazev CallListu mapy
int DLSubPlate;                             //Nazev CallListu podkladu mapy
std::list<sMapObjInfo> CPList;              //seznam objektu v mape



//operator pro usporadani pole od nejzazsiho po nejprednejsi prvek
bool operator <(const sMapObjInfo &a, const sMapObjInfo &b) {
    return a.y < b.y;
}


//generuje seznam pohyblivych objektu pro vnitrni pouziti CockpitMap;
int GenListofMobiles(scMobile * mList);

//generuje seznam statickych objektu pro vnitrni pouziti CockpitMap;
int GenListofStatics(scStatic * sList);

// prevede souradnici X do novych souradnic
int X2CockpitMap(int kvadx, int kvady);

// prevede souradnici Y do novych souradnic
int Y2CockpitMap(int kvady);

// vytvori displaylist pro mrizku mapy
int BuildCockpitMapField();



//------------------------------------------------------------------------------
//implementace exportovanych rutin CockpitMapy
//------------------------------------------------------------------------------


//inicializace cockpitmapy
int CockpitMapInit() {
	BuildCockpitMapField();
	CockpitMapChangeTransparency();
	// inicializacni nastaveni
	float r = guiInfo.winWidth/640;
	CockpitMapChangeScale(100*r);             //zmena rozliseni (zobrazeneho prostoru)
	CockpitMapChangePos(485*r, 90*r);         //zmena pozice
	CockpitMapChangeSize(200*r);              //zmena velikosti mapy
	return 1;
}


// vola metody a funkce pro vykresleni mapy do kokpitu
int CockpitMapDraw() {

	//vymazani seznamu vrcholu mapy
	CPList.clear();
	//vytvoreni iteratoru
	std::list<sMapObjInfo>::iterator listIt;
	//docasne promenne pro vypocet novych souradnic x a z
	int newX, newZ;
	//pomer pro nasobeni carky vysky/hloubky objektu od pozorovatele
	float HeightScale = 2*CMRdelta;

	//Vygenerovani seznamu vrcholu a jeho setrideni
	GenListofStatics(StaticList.First);
	GenListofMobiles(MobileList.First);
	CPList.sort();

	//pokud byla povolena pruhlednost, nic se pod mapu nekresli
	if (!CMTransparent) glCallList(DLSubPlate);

    //vykresli se objekty pod pozorovatelem (y < 0)
    glPointSize(3);
    for (listIt = CPList.begin(); listIt != CPList.end(); listIt++)
        if (listIt->y < 0)
        {
             newX = X2CockpitMap(listIt->x,listIt->z);
             newZ = Y2CockpitMap(listIt->z);
             glBegin(GL_LINES);
                 glColor3f(listIt->r,listIt->g,listIt->b);
                 glVertex2f(newX, newZ);
                 glVertex2f(newX, newZ+listIt->y/HeightScale);
             glEnd();
             glBegin(GL_POINTS);
                 glVertex2f(newX, newZ+listIt->y/HeightScale);
             glEnd();
        }


    //vykresli se mrizka
    glCallList(DLCockpitMap);

    //vykresli se body nad pozorovatelem (y >= 0)
    glPointSize(3);
    for (listIt = CPList.begin(); listIt != CPList.end(); listIt++)
        if (listIt->y >= 0)
        {
             newX = X2CockpitMap(listIt->x,listIt->z);
	     newZ = Y2CockpitMap(listIt->z);
	     glBegin(GL_LINES);
                 glColor3f(listIt->r,listIt->g,listIt->b);
                 glVertex2f(newX, newZ);
                 glVertex2f(newX, newZ+listIt->y/HeightScale);
             glEnd();
             glBegin(GL_POINTS);
                 glVertex2f(newX, newZ+listIt->y/HeightScale);
             glEnd();
        }
    return 0;
}



// zmena priblizeni v mape
int CockpitMapChangeScale(int cmscale) {
    CMRange = cmscale;
    CMRdelta = ((float) MapSize) / CMRange;      //prepocet delta pro zobrazeni v mape
    return 1;
}


// zmena velikosti mapy
int CockpitMapChangeSize(int newsize) {
    MapSize = newsize/2;                          //pocitame s 1/2 delky strany
    CMRdelta = ((float) MapSize) / CMRange;       //prepocet delta pro zobrazeni v mape
    glDeleteLists(DLCockpitMap, 1);
    glDeleteLists(DLSubPlate, 1);
    BuildCockpitMapField();
    return 1;
}


// zmena pozice mapy - ulozi nove X a Y
int CockpitMapChangePos(int newx, int newy) {
    MapXpos = newx;
    MapYpos = newy;
    return 1;
}


// zmena pruhlednosti mapy
int CockpitMapChangeTransparency() {
    CMTransparent = !CMTransparent;
    return 1;
}

//-----------------------------------------------------------------------
// implementace funkci pro CockpitMap
//-----------------------------------------------------------------------



// vytvori displaylist pro mrizku mapy a jeji mozny podklad, pokud ma byt nepruhledna
int BuildCockpitMapField() {
    DLCockpitMap = glGenLists(1);
    DLSubPlate   = glGenLists(1);

    glNewList(DLSubPlate, GL_COMPILE);
    glBegin(GL_QUADS);
        glColor3f(0.0f, 0.0f, 0.0f);
        glVertex2f(X2CockpitMap( MapSize, MapSize), Y2CockpitMap( MapSize));
        glVertex2f(X2CockpitMap(-MapSize, MapSize), Y2CockpitMap( MapSize));
        glVertex2f(X2CockpitMap(-MapSize,-MapSize), Y2CockpitMap(-MapSize));
        glVertex2f(X2CockpitMap( MapSize,-MapSize), Y2CockpitMap(-MapSize));
    glEnd();
    glEndList();


    glNewList(DLCockpitMap, GL_COMPILE);
    glBegin(GL_LINES);
        glColor3f(0.5f,0.5f,0.5f);

        //stredni svisla cara
        glVertex2f(X2CockpitMap(0 , MapSize), Y2CockpitMap( MapSize));
        glVertex2f(X2CockpitMap(0 ,-MapSize), Y2CockpitMap(-MapSize));

        //stredni vodorovna cara
        glVertex2f(X2CockpitMap(-MapSize, 0), MapYpos);
        glVertex2f(X2CockpitMap( MapSize, 0), MapYpos);

        //horni okraj mapy
        glVertex2f(X2CockpitMap(-MapSize, MapSize), Y2CockpitMap( MapSize));
        glVertex2f(X2CockpitMap( MapSize, MapSize), Y2CockpitMap( MapSize));
        //dolni okraj mapy
        glVertex2f(X2CockpitMap(-MapSize,-MapSize), Y2CockpitMap(-MapSize));
        glVertex2f(X2CockpitMap( MapSize,-MapSize), Y2CockpitMap(-MapSize));
        //levy okraj mapy
        glVertex2f(X2CockpitMap(-MapSize, MapSize), Y2CockpitMap( MapSize));
        glVertex2f(X2CockpitMap(-MapSize,-MapSize), Y2CockpitMap(-MapSize));
        //pravy okraj mapy
        glVertex2f(X2CockpitMap(+MapSize, MapSize), Y2CockpitMap( MapSize));
        glVertex2f(X2CockpitMap(+MapSize,-MapSize), Y2CockpitMap(-MapSize));

        //horni pulici cara
        glVertex2f(X2CockpitMap(-MapSize, MapSize/2), Y2CockpitMap(MapSize/2));
        glVertex2f(X2CockpitMap( MapSize, MapSize/2), Y2CockpitMap(MapSize/2));
        //dolni pulici cara
        glVertex2f(X2CockpitMap(-MapSize,-MapSize/2), Y2CockpitMap(-MapSize/2));
        glVertex2f(X2CockpitMap( MapSize,-MapSize/2), Y2CockpitMap(-MapSize/2));
        //leva pulici cara
        glVertex2f(X2CockpitMap(-MapSize/2, MapSize), Y2CockpitMap( MapSize));
        glVertex2f(X2CockpitMap(-MapSize/2,-MapSize), Y2CockpitMap(-MapSize));
        //prava pulici cara
        glVertex2f(X2CockpitMap( MapSize/2, MapSize), Y2CockpitMap( MapSize));
        glVertex2f(X2CockpitMap( MapSize/2,-MapSize), Y2CockpitMap(-MapSize));
    glEnd();

    glPointSize(3);
    glBegin(GL_POINTS);
        glColor3f(1.0f,0.0f,1.0f);
        glVertex2f(X2CockpitMap(0,0), Y2CockpitMap(0));
    glEnd();
    glEndList();

    return 1;
}



// prevede souradnici X do novych souradnic
int X2CockpitMap(int kvadx, int kvady) {
    if (kvady > 0) return MapXpos + kvadx + kvady*kvadx*deltaRy;
    if (kvady < 0) return MapXpos + kvadx + abs(kvady)*kvadx*deltaFy;
    return MapXpos + kvadx;
}



// prevede souradnici Y do novych souradnic
int Y2CockpitMap(int kvady) {
    return MapYpos + kvady*MapSlope;
}



//generuje seznam statickych objektu pro vnitrni pouziti CockpitMap;
int GenListofStatics(scStatic * sList) {
    sMapObjInfo tmpVect,tgtVect;          //docasne vektory

    tgtVect.r = 0.0;                      //ulozeni barev pro staticky objekt
    tgtVect.g = 1.0;
    tgtVect.b = 0.0;

    while (sList!=NULL)
    {
		tmpVect.x = sList->Matrix[3][0] - ViewersMatrices.ViewerPos[0];        //posun do (0,0,0)
		tmpVect.y = sList->Matrix[3][1] - ViewersMatrices.ViewerPos[1];
		tmpVect.z = sList->Matrix[3][2] - ViewersMatrices.ViewerPos[2];

		//pokud je objekt na mape viditelny, provede se s nim maticove nasobeni s transponovanou
		//rot matici, aby do sebe padly osy kamery a mapy
		if ((abs(tmpVect.x) < CMRange)&&(abs(tmpVect.y) < CMRange)&&(abs(tmpVect.z) < CMRange))
		{
			 tgtVect.x = + tmpVect.x*ViewersMatrices.ViewerRotMatrix[0][0]
						 + tmpVect.y*ViewersMatrices.ViewerRotMatrix[1][0]
						 + tmpVect.z*ViewersMatrices.ViewerRotMatrix[2][0];

			 tgtVect.y = + tmpVect.x*ViewersMatrices.ViewerRotMatrix[0][1]
						 + tmpVect.y*ViewersMatrices.ViewerRotMatrix[1][1]
						 + tmpVect.z*ViewersMatrices.ViewerRotMatrix[2][1];

			 tgtVect.z = - tmpVect.x*ViewersMatrices.ViewerRotMatrix[0][2]     //kamera se diva do -z, na mape vsak do +z
						 - tmpVect.y*ViewersMatrices.ViewerRotMatrix[1][2]     //proto jsou hodnoty nasobeny (-1)
						 - tmpVect.z*ViewersMatrices.ViewerRotMatrix[2][2];

			 tgtVect.x *= CMRdelta;                                        //nasobeno pomerem Prostor => CockpitMap
			 tgtVect.y *= CMRdelta;
			 tgtVect.z *= CMRdelta;

			 //opetna kontrola, jestli po nasobeni pomerem nektere objekty po otoceni nejsou mimo mapu
			 if ((abs(tgtVect.x) < MapSize)&&(abs(tgtVect.y) < MapSize)&&(abs(tgtVect.z) < MapSize))
				  CPList.push_back(tgtVect);
		}
		sList = (scStatic*)sList->Next;
	}

	return 1;
}



//generuje seznam pohyblivych objektu pro vnitrni pouziti CockpitMap;
int GenListofMobiles(scMobile * mList) {
	sMapObjInfo tmpVect, tgtVect;

	tgtVect.r = 1.0;
	tgtVect.g = 0.0;
	tgtVect.b = 0.0;

    while (mList!=NULL)
    {
		tmpVect.x = mList->Matrix[3][0] - ViewersMatrices.ViewerPos[0];        //posun do (0,0,0)
		tmpVect.y = mList->Matrix[3][1] - ViewersMatrices.ViewerPos[1];
		tmpVect.z = mList->Matrix[3][2] - ViewersMatrices.ViewerPos[2];

		//pokud je objekt na mape viditelny, provede se s nim maticove nasobeni s transponovanou
		//rot matici, aby do sebe padly osy kamery a mapy
		if ((abs(tmpVect.x) < CMRange)&&(abs(tmpVect.y) < CMRange)&&(abs(tmpVect.z) < CMRange))
		{
			 tgtVect.x = + tmpVect.x*ViewersMatrices.ViewerRotMatrix[0][0]
						 + tmpVect.y*ViewersMatrices.ViewerRotMatrix[1][0]
						 + tmpVect.z*ViewersMatrices.ViewerRotMatrix[2][0];

			 tgtVect.y = + tmpVect.x*ViewersMatrices.ViewerRotMatrix[0][1]
						 + tmpVect.y*ViewersMatrices.ViewerRotMatrix[1][1]
						 + tmpVect.z*ViewersMatrices.ViewerRotMatrix[2][1];

			 tgtVect.z = - tmpVect.x*ViewersMatrices.ViewerRotMatrix[0][2]     //kamera se diva do -z, na mape vsak do +z
						 - tmpVect.y*ViewersMatrices.ViewerRotMatrix[1][2]     //proto jsou hodnoty nasobeny (-1)
						 - tmpVect.z*ViewersMatrices.ViewerRotMatrix[2][2];

			 tgtVect.x *= CMRdelta;
			 tgtVect.y *= CMRdelta;
			 tgtVect.z *= CMRdelta;

			 //opetna kontrola, jestli po nasobeni pomerem nektere objekty po otoceni nejsou mimo mapu
			 if ((abs(tgtVect.x) < MapSize)&&(abs(tgtVect.y) < MapSize)&&(abs(tgtVect.z) < MapSize))
				 CPList.push_back(tgtVect);
		}
		mList = (scMobile*)mList->Next;
	}
	return 1;
}
