/*
 * The POG project
 * Copyright 2000 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */


#include <string.h>
#include <assert.h>

#define WANT_NEW
#include "porting.h"

#include "mygl.h"
#include "Fonts.h"
#include "font_Fotinos19.h"



BitmapFont *font_Fotinos19 = NULL;



int FontsInit(void)
{
  font_Fotinos19 = new(nothrow) BitmapFont(&font_Fotinos19_Data);
	if (font_Fotinos19 == NULL)
	{ fprintf(stderr, "Fonts error: Out of memory."); return 0; }

	return 1;
}


void FontsDone(void)
{
	if (font_Fotinos19 != NULL)
		delete font_Fotinos19;
}




inline GLboolean GetLighting(void)
{
	GLboolean L;
	glGetBooleanv(GL_LIGHTING, &L);
	return L;
}


void BitmapFont::DrawString(char *S, GLubyte *Color,
		const GLfloat X, const GLfloat Y,	const GLfloat Z)
{
	assert(!GetLighting()); // lighting must be off if we use glColor, otherwise we must use different access (glMaterial)

	glColor3ubv(Color);
	glRasterPos3f(X, Y, Z);
	glPushAttrib(GL_LIST_BIT);
	glListBase(lists);
	glCallLists(strlen(S), GL_UNSIGNED_BYTE, (GLubyte *)S);
	glPopAttrib();
}


void BitmapFont::Init(BitmapFontData *Data)
{
	int i;

	Done();

	width = Data->width;
	height = Data->height;
	lists = glGenLists(BitmapFontMaxChars);

	for (i = 0; i < BitmapFontMaxChars; i++) {
		if (Data->data[i].width) {
			glNewList(lists + i, GL_COMPILE);
			glBitmap(Data->data[i].width, Data->height, 0.0, 0.0, (GLfloat)Data->data[i].width, 0.0, (unsigned char*)Data->data[i].map);
			glEndList();
		}
	}
}


void BitmapFont::Done()
{
	if (lists!=0)
	{
		glDeleteLists(lists, BitmapFontMaxChars);
		lists = 0;
	}
}



