/*
 * The POG project
 * Copyright 2000 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */

#ifndef _Input_h_
#define _Input_h_


int InputInit(void);
void InputDone(void);
// InputUpdate is called for update structures in this module
int InputUpdate(void);


// InputAcquire - hides mouse cursor and connects all devices (curerently
// mouse, keyboard) to application
void InputAcquire(void);
// InputUnacquire - releases all input devices and shows mouse cursor
void InputUnacquire(void);


// InputAxesTable - variables of linear user input devices
typedef struct
{
	float RotLR;    // rotation left-right - axes y
	float RotDU;    // rotation down-up - axes x
	float Roll;     // rotation rolling - axes z
	float MovForward; // movement forward - axes z
	float MovLR;    // movement left-right - axes x
	float MovDU;    // movement down-up - axes y
} InputAxesTable;

extern InputAxesTable InputAxes;


typedef struct {
	bool Down;
	bool Pressed;
	int Mod;
} InputKeyInfo;
// InputKeysTable - variables of key user input devices
typedef struct
{
	bool Fire;      // rocket fire
	bool Pause;
	bool Menu;

	bool ChangeCamera;
	bool ArrowUp;   // for menu
	bool ArrowDown; // for menu
	bool FPSKey;    // show/hide FPS
	bool Terminate; // terminate application
} InputKeysTable;

// InputIsDown - indicates that key is down
extern InputKeysTable InputKeys;
// InputIsPressed - indicates that the user push key down just now.
// Any of bools in this structure is set to true only once per key
// press/release interval opposite to InputIsDown which is true for whole
// press/release interval.
//extern InputKeysTable InputIsPressed;


void InputAssignKeyDown(const int Ch, const bool *b);
void InputAssignKeyPress(const int Ch, const bool *b);


/*	InputAssignment RotL;
	InputAssignment RotR;
	InputAssignment RotD;
	InputAssignment RotU;
	InputAssignment RolL;
	InputAssignment RolR;
	InputAssignment MovF;
	InputAssignment MovB;
	InputAssignment MovR;
	InputAssignment MovL;
	InputAssignment MovU;
	InputAssignment MovD;

	InputAssignment Fire;
	InputAssignment ChangeCamera;
	InputAssignment Menu;
	InputAssignment Pause;

	InputAssignment ArrowUp;   // for menu
	InputAssignment ArrowDown; // for menu
	InputAssignment FPSKey;

	InputAssignment Terminate;*/


#endif /* _Input_h_ */

