/*
 * The POG project
 * Copyright 2000 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */


#include <stdio.h>
#include "InputDevices.h"
#include "gui.h"

#include "setup.h"

#ifdef USE_SDL
# include <SDL/SDL_mouse.h>
# include <SDL/SDL_keyboard.h>
#else
# ifdef USE_GLUT
#  undef GLUT_BUILDING_LIB
#  undef GLUT_API_VERSION 4
#  define GLUT_API_VERSION 4
#  include <GL/glut.h>
# endif
#endif

static int ZeroPosX;
static int ZeroPosY;
static bool MouseAcquired = false;

#ifdef USE_GLUT
static int MouseX;
static int MouseY;
static void cbMouseMotion(int x, int y);
static void cbMouseButton(int button, int state, int x, int y);
#endif



int MouseGetData(int &X, int &Y)
{
#ifdef USE_SDL
	int x,y;
	SDL_GetMouseState(&x,&y);
    X = x-ZeroPosX;
    Y = y-ZeroPosY;
    SDL_WarpMouse(ZeroPosX,ZeroPosY);
//    int xt,yt;
//	SDL_GetRelativeMouseState(&xt,&yt);
    return 1;
#else
# ifdef USE_GLUT
	X = MouseX;
	Y = MouseY;
	MouseX = 0;
	MouseY = 0;
	return 1;
# endif
#endif
/*	POINT p;
	int mx = glutGet(GLUT_WINDOW_X) + guiInfo.winWidth2;
	int my = glutGet(GLUT_WINDOW_Y) + guiInfo.winHeight2;

	if (!GetCursorPos(&p)) return 0;
	X = p.x - mx;
	Y = p.y - my;

	if (!SetCursorPos(mx, my)) return 0;

	return 1;
#endif*/
}


int MouseInit(void)
{
#ifdef USE_SDL
	return 1;
#else
# ifdef USE_GLUT
	return glutDeviceGet(GLUT_HAS_MOUSE);
# endif
#endif
}


void MouseDone(void)
{
	MouseUnacquire();
}


int MouseWindowCallback(void)
{
	if (MouseAcquired)
	{
	#ifdef USE_SDL
		ZeroPosX = guiInfo.winWidth2;
		ZeroPosY = guiInfo.winHeight2;
	    SDL_WarpMouse(ZeroPosX,ZeroPosY);
		int x,y;
		SDL_GetMouseState(&x,&y);
    #endif
	#ifdef USE_GLUT
		int wx = glutGet(GLUT_WINDOW_X);
		int wy = glutGet(GLUT_WINDOW_Y);
		ZeroPosX = wx+guiInfo.winWidth2;
		ZeroPosY = wy+guiInfo.winHeight2;
/*		RECT r = { wx, wy, wx+guiInfo.winWidth, wy+guiInfo.winHeight };
		if (!ClipCursor(&r)) return 0;
		if (!SetCursorPos(wx+guiInfo.winWidth2, wy+guiInfo.winHeight2)) return 0;*/
		glutWarpPointer(ZeroPosX, ZeroPosY);
	#endif
	}
	return 1;
}


int MouseAcquire(void)
{
	if (!MouseAcquired)
	{
    int r;
    #ifdef USE_SDL
        SDL_ShowCursor(0);
		#ifdef DEBUG
		SDL_WM_GrabInput(SDL_GRAB_OFF);
		#else
		SDL_WM_GrabInput(SDL_GRAB_ON);
		#endif
	#endif
		MouseAcquired = true;
		r = MouseWindowCallback();
	#ifdef USE_GLUT
		glutWarpPointer(ZeroPosX, ZeroPosY);
		glutMotionFunc(cbMouseMotion);
		glutPassiveMotionFunc(cbMouseMotion);
		glutMouseFunc(cbMouseButton);
		MouseX = 0;
		MouseY = 0;
	#endif
    return r;
	} else return 1;
}


void MouseUnacquire(void)
{
	if (MouseAcquired)
	{
		MouseAcquired = false;
//		ClipCursor(NULL);
	#ifdef USE_SDL
		SDL_WarpMouse(ZeroPosX,ZeroPosY);
		SDL_WM_GrabInput(SDL_GRAB_OFF);
		SDL_ShowCursor(1);
	#endif
	#ifdef USE_GLUT
		glutMotionFunc(NULL);
		glutPassiveMotionFunc(NULL);
		glutMouseFunc(NULL);
		glutWarpPointer(ZeroPosX, ZeroPosY);
		MouseX = 0;
		MouseY = 0;
	#endif
	}
}


#ifdef USE_GLUT
static void cbMouseMotion(int x, int y)
{
	if ((x!=ZeroPosX) || (y!=ZeroPosY))
	{
		MouseX += x-ZeroPosX;
		MouseY += y-ZeroPosY;
		glutWarpPointer(ZeroPosX, ZeroPosY);
	}
}

static void cbMouseButton(int button, int state, int x, int y)
{
}
#endif




static bool KeyboardAcquired = false;

KeyboardKeyInfo KeyboardKeyState[KEYBOARD_KEY_MAX];

#ifdef USE_GLUT
char KeyboardBuffer[256];
static int IgnoreState;
static void cbKeyDown(unsigned char key, int x, int y);
static void cbKeyUp(unsigned char key, int x, int y);
static void cbSpecialKeyDown(int key, int x, int y);
static void cbSpecialKeyUp(int key, int x, int y);
#endif


void KeyboardKeyDownCallback(const int Ch, const int Mod)
{
	KeyboardKeyState[Ch].Down = true;
	KeyboardKeyState[Ch].Pressed = true;
	KeyboardKeyState[Ch].Mod = Mod;
}


void KeyboardKeyUpCallback(const int Ch)
{
	KeyboardKeyState[Ch].Down = false;
	KeyboardKeyState[Ch].Mod = 0;
}


void KeyboardClearPressed()
{
	for(int i=0; i<KEYBOARD_KEY_MAX; i++)
	{
		KeyboardKeyState[i].Pressed = false;
	}
}


int KeyboardUpdate(void)
{
	return 1;
}


int KeyboardInit(void)
{
#ifdef USE_SDL
	return 1;
#endif
#ifdef USE_GLUT
	return glutDeviceGet(GLUT_HAS_KEYBOARD);
#endif
}


void KeyboardDone(void)
{
    KeyboardUnacquire();
}


int KeyboardAcquire(void)
{
	if (!KeyboardAcquired)
	{
	#ifdef USE_GLUT
		glutKeyboardFunc(cbKeyDown);
		glutSpecialFunc(cbSpecialKeyDown);
		glutKeyboardUpFunc(cbKeyUp);
		glutSpecialUpFunc(cbSpecialKeyUp);
		IgnoreState = glutDeviceGet(GLUT_DEVICE_IGNORE_KEY_REPEAT);
		glutIgnoreKeyRepeat(1);
	#endif
		KeyboardAcquired = true;
	}
	return 1;
}


void KeyboardUnacquire(void)
{
	if (KeyboardAcquired)
	{
		KeyboardAcquired = false;
	#ifdef USE_GLUT
		glutKeyboardFunc(NULL);
		glutSpecialFunc(NULL);
		glutKeyboardUpFunc(NULL);
		glutSpecialUpFunc(NULL);
	#endif
	}
}


#ifdef USE_GLUT
static void cbKeyDown(unsigned char key, int x, int y)
{
	KeyboardBuffer[key] |= 0x80;

	switch (key) {
		case 27 :
		case 'q' : case 'Q' :
			guiTerminateApp();
    		break;
	}
}

static void cbKeyUp(unsigned char key, int x, int y)
{
	KeyboardBuffer[key] &= 0x7f;
}

static void cbSpecialKeyDown(int key, int x, int y)
{
	if ((key<=255) && (key>=0))
		KeyboardBuffer[key] |= 0x80;
}

static void cbSpecialKeyUp(int key, int x, int y)
{
	if ((key<=255) && (key>=0))
		KeyboardBuffer[key] &= 0x7f;
}
#endif


