/*
 * The POG project
 * Copyright 2000 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */

#ifndef _Errors_h_
#define _Errors_h_



#define LOG_EXINFO_ENABLED
#define LOG_INFO_ENABLED
#define LOG_WARNINGS_ENABLED
#define LOG_ERRORS_ENABLED




enum logModule { modErrors=0, modMain, modGUI, modTimers, modFonts,
                 modCockpit, modInput, mod3dsLoad, modScene,
                 modSceneClasses, modTextures, modTgaPcxLoad,
                 modAI, /*next modules insert here*/
                 modUnknown }; // modUnknown must be last

enum logLevel { logLevelFatalErrors=0, logLevelErrors, logLevelWarnings, logLevelInfos, logLevelExInfos };

extern logLevel logLevelTable[modUnknown+1];
extern bool logFatalErrorsFound;


int logInit(void);
int logDone(void);
void logShow(void);

void logFatalError(logModule M, char *S, ...);
void logError(logModule M, char *S, ...);
void logWarning(logModule M, char *S, ...);
void logInfo(logModule M, char *S, ...);
void logExInfo(logModule M, char *S, ...);



#ifdef LOG_ERRORS_ENABLED
void logError(logModule M, char *S, ...);
#else
inline void logError(logModule M, char *S, ...)  {};
#endif

#ifdef LOG_WARNINGS_ENABLED
void logWarning(logModule M, char *S, ...);
#else
inline void logWarning(logModule M, char *S, ...)  {};
#endif

#ifdef LOG_INFO_ENABLED
void logInfo(logModule M, char *S, ...);
#else
inline void logInfo(logModule M, char *S, ...)  {};
#endif

#ifdef LOG_EXINFO_ENABLED
void logExInfo(logModule M, char *S, ...);
#else
inline void logExInfo(logModule M, char *S, ...)  {};
#endif


#endif /* _Errors_h_ */
