/*
 * The POG project
 * Copyright 2000 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */


#include <math.h>
#include <stdlib.h> //jen pro ucely testovani (nahodne cisla)
#define WANT_NEW
#define WANT_DISABLE_MATH_EXCEPTIONS
#define WANT_SQR
#include "porting.h"
#pragma hdrstop // for C++ Builder compiler speed up
#include "SceneClasses.h"
#include "Scene.h"
#include "3dsLoad.h"
#include "gui.h"
#include "dlist.h"
#include "CockpitMap.h"
#include "RocketSmoke.h"
#include "Cameras.h"
#include "Background.h"
#include "BillExp.h"
#include "Input.h"
//#include "ArtificialIntelligence.h"

#include "setup.h"



char ScenePath[] = "scene\\";
FreezeMode FreezeStatus = fsPaused;
int StartCounter;
#define StartFramesCount 3


// functions in this module
int SceneInit(void);
void SceneDone(void);
void SceneTimeTick(const float DTime);
void SceneRender(void);
void SceneClear(void);
int SceneLoadDefault(void);
static int MakeDefaultScene(void);
static int MakeSimpleModel(void);
FreezeMode SceneFreezeStatus(void);
void SceneFreeze(bool F);
int IsCrush(scStatic *A, scStatic *B);
int IsCollision(scStatic *A);
int Insert(scStatic *A);
void Collision(void);


// StaticList - double-linked list of static scene objects
XList<scStatic> StaticList;
// MobileList - double-linked list of mobile scene objects
XList<scMobile> MobileList;
// BlendList - double-linked list of scene objects using blending
XList<scObject> BlendList;

// pre-loaded display lists for scene objects
#ifdef USEMOD_SCENE
struct {
	GLint F1;
	GLint F2;
	GLint Missile;
	GLint Earth;
	GLint RedPlanet;
	GLint BluePlanet;
	GLint Sun;
} DisplayLists = {0,0,0,0,0,0,0};
#endif



int SceneInit(void)
{
#ifdef USEMOD_SCENE
	ThisPlayer = new(nothrow) scRocket;
	if (ThisPlayer == NULL) return 0;
#endif

	if (!SceneLoadDefault()) return 0;

	MainCamera.Set(ThisPlayer);
	Textures();
	return 1;
}


void SceneDone(void)
{
#ifdef USEMOD_SCENE
	delete ThisPlayer;
#endif
}


static void DoTimeTick(const float DTime)
{
// static objects don't need to call TimeTick()
//	scStatic *S = StaticList.First;
//	while (S!=NULL) {
//		S->TimeTick(DTime);
//		S = (scStatic*)S->Succ;
//	}

	// rocket firing
	if (InputKeys.Fire) {
		scRocket *R = new(nothrow) scRocket;
		if (R)
		{
			R->DisplayList = DisplayLists.Missile;
			R->ColRadius = 5;
			MatrixCopy(&R->Matrix, &ThisPlayer->Matrix);
			R->SetPosition(ThisPlayer->GetPosX() + ThisPlayer->GetDirX() * 21 - ThisPlayer->GetUpX() * 10,
					ThisPlayer->GetPosY() + ThisPlayer->GetDirY() * 21 - ThisPlayer->GetUpY() * 10,
					ThisPlayer->GetPosZ() + ThisPlayer->GetDirZ() * 21 - ThisPlayer->GetUpZ() * 10);
			R->SetSpeed(ThisPlayer->Speed.x + ThisPlayer->GetDirX() * 10,
					ThisPlayer->Speed.y + ThisPlayer->GetDirY() * 10,
					ThisPlayer->Speed.z + ThisPlayer->GetDirZ() * 10);
			MobileList.Insert(R);

			RocketSmoke *S = new(nothrow) RocketSmoke;
			if (S)
			{
				S->Assign(R,-5);
				BlendList.Insert(S);
			}
		}
	}

	scMobile *M = MobileList.First;
	while (M!=NULL) {
		M->TimeTick(DTime);
		M = (scMobile*)M->Next;
	}

	scObject *O = BlendList.First;
	while (O!=NULL) {
		O->TimeTick(DTime);
		O = (scStatic*)O->Next;
	}

	Collision();
}


void SceneTimeTick(const float DTime)
{
	if (FreezeStatus == fsRunning)  DoTimeTick(DTime);
	else
		if (FreezeStatus == fsStarting)
		{
			StartCounter++;
			if (StartCounter == StartFramesCount)
			{
				FreezeStatus = fsRunning;
				DoTimeTick(DTime);
			}
			else
				DoTimeTick(0);
		}
		else
			DoTimeTick(0);
}


void SceneRender(void)
{
#ifndef USEMOD_SCENE
	glLoadIdentity();
#endif

	// Borland needs disable math exception during rendering through OpenGL
	Disable8087Exceptions();

	// remove ThisPlayer from the scene
	MobileList.Delete(ThisPlayer);

	// scene rendering ...
	BackgroundRender();

	scStatic *S = StaticList.First;
	while (S!=NULL) {
		S->Render();
		S = (scStatic*)S->Next;
	}

	scMobile *M = MobileList.First;
	while (M!=NULL) {
		M->Render();
		M = (scMobile*)M->Next;
	}

	scObject *O = BlendList.First;
	while (O!=NULL) {
		O->Render();
		O = (scObject*)O->Next;
	}

	// enable math exceptions for Borland
	Enable8087Exceptions();

	// add ThisPlayer back to the scene
	MobileList.Insert(ThisPlayer);
}


void SceneClear(void)
{
	scStatic *S = StaticList.First;
	scStatic *TS;
	while (S!=NULL) {
		TS = (scStatic*)S->Next;
		delete S;
		S = TS;
	}
	StaticList.First = NULL;
	StaticList.Last = NULL;

	scMobile *M = MobileList.First;
	scMobile *TM;
	while (M!=NULL) {
		TM = (scMobile*)M->Next;
		delete M;
		M = TM;
	}
	MobileList.First = NULL;
	MobileList.Last = NULL;

	glDeleteLists(DisplayLists.F1,1);
	glDeleteLists(DisplayLists.F2,1);
	glDeleteLists(DisplayLists.Missile,1);
	glDeleteLists(DisplayLists.Earth,1);
	glDeleteLists(DisplayLists.RedPlanet,1);
	glDeleteLists(DisplayLists.BluePlanet,1);
	glDeleteLists(DisplayLists.Sun,1);
	DisplayLists.F1 = DisplayLists.F2 = DisplayLists.Missile = DisplayLists.Earth = 0;
	DisplayLists.RedPlanet = DisplayLists.BluePlanet = DisplayLists.Sun = 0;

}


int SceneLoadDefault(void)
{
	SceneClear();
#ifdef USEMOD_SCENE
	MatrixLoadIdentity(&ThisPlayer->Matrix);
#endif
	return MakeDefaultScene();
}

inline double randomin(double min, double max)
{
  return (max-min)*(double(rand())/RAND_MAX) + min;
}

inline void RandomizeRotation(scStatic *X)
{
  X->Rotate(randomin(-M_PI, M_PI),
            randomin(-M_PI, M_PI),
            randomin(-M_PI, M_PI));
}
inline void RandomizeSpeed(scMobile *X, double MaxSpeed)
{
  X->SetSpeed(randomin(-MaxSpeed/2, MaxSpeed/2),
              randomin(-MaxSpeed/2, MaxSpeed/2),
              randomin(-MaxSpeed/2, MaxSpeed/2));
}
inline void RandomizePosition(scStatic *X, double MaxPosition)
{
  X->SetPosition(randomin(-MaxPosition/2, MaxPosition/2),
                 randomin(-MaxPosition/2, MaxPosition/2),
                 randomin(-MaxPosition/2, MaxPosition/2));
}

static int MakeDefaultScene(void)
{
#ifdef USEMOD_SCENE
//	if (!AITest()) return 0;

	if (DisplayLists.F1 == 0)
	{
		 DisplayLists.F1 = Load3DS("Fighter1.3ds");
		 if (DisplayLists.F1 == 0) return 0;
	}
	if (DisplayLists.F2 == 0)
	{
		DisplayLists.F2 = Load3DS("Fighter2.3ds");
		if (DisplayLists.F2 == 0) return 0;
	}
	if (DisplayLists.Missile == 0)
	{
		DisplayLists.Missile = Load3DS("maa-1.3ds");
		if (DisplayLists.Missile == 0) return 0;
	}
	if (DisplayLists.Earth == 0)
	{
		DisplayLists.Earth = Load3DS("EarthHi.3ds");
		if (DisplayLists.Earth == 0) return 0;
	}
	if (DisplayLists.RedPlanet == 0)
	{
		DisplayLists.RedPlanet = Load3DS("RedPlanet.3ds");
		if (DisplayLists.RedPlanet == 0) return 0;
	}
	if (DisplayLists.BluePlanet == 0)
	{
		DisplayLists.BluePlanet = Load3DS("BluePlanet.3ds");
		if (DisplayLists.BluePlanet == 0) return 0;
	}
	if (DisplayLists.Sun == 0)
	{
		DisplayLists.Sun = Load3DS("Sun.3ds");
		if (DisplayLists.Sun == 0) return 0;
	}

	const float MaxSpeed=10;
	ThisPlayer->DisplayList = DisplayLists.F1;
	ThisPlayer->ColRadius = 10;
	MobileList.Insert(ThisPlayer);

# ifndef COLLISION_TEST
	// F1 team
	scRocket *R = new scRocket;
	R->DisplayList = DisplayLists.F1;
	R->ColRadius = 10;
	RandomizeRotation(R);
	RandomizeSpeed(R, MaxSpeed);
	if (Insert(R)) MobileList.Insert(R);

	R = new scRocket;
	R->DisplayList = DisplayLists.F1;
	R->ColRadius = 10;
	RandomizeRotation(R);
	RandomizeSpeed(R, MaxSpeed);
	if (Insert(R)) MobileList.Insert(R);

	// F2 team
	R = new scRocket;
	R->DisplayList = DisplayLists.F2;
	R->ColRadius = 10;
	RandomizeRotation(R);
	RandomizeSpeed(R, MaxSpeed);
	if (Insert(R)) MobileList.Insert(R);

	R = new scRocket;
	R->DisplayList = DisplayLists.F2;
	R->ColRadius = 10;
	RandomizeRotation(R);
	RandomizeSpeed(R, MaxSpeed);
	if (Insert(R)) MobileList.Insert(R);

	// static scene
	scStatic *S = new scStatic;
	S->DisplayList = DisplayLists.Sun;
	S->ColRadius = 60;
	RandomizeRotation(S);
	if (Insert(S)) StaticList.Insert(S);

	S = new scStatic;
	S->DisplayList = DisplayLists.Earth;
	S->ColRadius = 20;
	RandomizeRotation(S);
	if (Insert(S)) StaticList.Insert(S);

	int i;
	for(i=0; i<5; i++)
	{
		S = new scStatic;
		S->DisplayList = DisplayLists.RedPlanet;
		S->ColRadius = 12;
		RandomizeRotation(S);
		if (Insert(S)) StaticList.Insert(S);
	}

	for(i=0; i<3; i++)
	{
		S = new scStatic;
		S->DisplayList = DisplayLists.BluePlanet;
		S->ColRadius = 32;
		RandomizeRotation(S);
		if (Insert(S)) StaticList.Insert(S);
	}

# else
	int i;
	for (i=0; i<30; i++) {  //30 statickch objektu
			scStatic *S = new scStatic;
			S->DisplayList = DisplayLists.RedPlanet;
			S->ColRadius = 12;
			if (Insert(S)) StaticList.Insert(S);
	}
	for (i=0; i<10; i++) { //10 pohyblivch objektu
			scRocket *R = new scRocket;
			R->DisplayList = DisplayLists.F1;
			R->ColRadius = 10;
			RandomizeSpeed(R);
			if (Insert(R)) MobileList.Insert(R);
	}
# endif

#else
	MakeSimpleModel();
#endif
	return 1;
}


//vrati 1 jde-li vlozit objekt do sceny
int Insert(scStatic *A)
{
	int i=0;
	const float Max = 202; //koule o prumeru Max
	const int MaxTest = 10; //MaxTest zkousek vlozeni pro kazdy objekt
	while (i<MaxTest) {
		RandomizePosition(A, Max); //nahodna pozice
		if (!(IsCollision(A))) return 1; //nekoliduje
		i++;
	}
	return 0;  //koliduje nevklada se nic
}

//vrati 1 dojde-li ke kolizi objektu
int IsCollision(scStatic *A)
{
	//mobilni objekty
	scStatic *TempM = MobileList.First;
	while (TempM!=NULL) {
		if ((TempM!=A) && IsCrush(A, TempM)) return 1; //je srazka
		TempM = (scMobile*)TempM->Next;
	}
	//staticke objekty
	scStatic *S = StaticList.First;
	while (S!=NULL) {
		if (IsCrush(A, S)) return 1; //je srazka
		S = (scStatic*)S->Next;
	}
	return 0;  //neni srazka
}

//vrati 1 doslo-li ke srazce objektu
int IsCrush(scStatic *A, scStatic *B)
{
	#define sqr(x) ((x)*(x))
	GLfloat suma=0.0;
	suma=(sqr((A->GetPosX() - B->GetPosX()))+
		  sqr((A->GetPosY() - B->GetPosY()))+
		  sqr((A->GetPosZ() - B->GetPosZ())));
	return (suma <= sqr(A->ColRadius + B->ColRadius));
}

void Collision(void)
{
#ifdef USEMOD_COLLISIONS
	scMobile *M = MobileList.First;
	while (M!=NULL) {
		GLint Move = ((M->Speed.x != 0.0) ||
				(M->Speed.y != 0.0) ||
				(M->Speed.z != 0.0));
		if (Move && IsCollision(M)) {
			M->SetSpeed(0,0,0);

			BillboardExplosions *E = new(nothrow) BillboardExplosions(M->GetPosX(), M->GetPosY(), M->GetPosZ());
			if (E!=NULL)
				BlendList.Insert(E);
		}
		M = (scMobile*)M->Next;
	}
#endif
}


static int MakeSimpleModel(void)
{
#define D 5.0
#define S 2.0
	int DL = glGenLists(1);
	glNewList(DL, GL_COMPILE);
		glBegin(GL_POLYGON);
		glColor3f(0.0,1.0,0.0);  glVertex3f(-S,-S,-D);
		glColor3f(0.0,0.0,1.0);  glVertex3f(+S,-S,-D);
		glColor3f(1.0,0.0,0.0);  glVertex3f(+S,+S,-D);
		glColor3f(1.0,1.0,0.0);  glVertex3f(-S,+S,-D);
		glEnd();
	glEndList();
#undef S
#undef D

	scMobile *M = new scMobile;
	M->DisplayList = DL;
	MobileList.Insert(M);

	return guiCheckForGLErrors(modScene, "Can't compile default scene to OpenGL (cube model), error(s): %s");
}



FreezeMode SceneFreezeStatus(void)
{
	return FreezeStatus;
}

void SceneFreeze(bool F)
{
	if (F)
	{
		if (FreezeStatus != fsPaused)
		{
			FreezeStatus = fsPaused;
		}
	}
	else
	{
		if (FreezeStatus == fsPaused)
		{
			FreezeStatus = fsStarting;
			StartCounter = 0;
		}
	}
}

