/*
 * The POG project
 * Copyright 2001 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */


#ifndef _dlist_h_
#define _dlist_h_

// template class for double-linked lists of scene objects
template <class T> class XList
{
public:
	T *First;
	T *Last;

	XList() { First = NULL; Last = NULL; }
	void Insert(T *S)
	{
		if (First==NULL)
		{
			S->Prev = NULL;
			S->Next = NULL;
			First = S;
			Last = S;
		}
		else
		{
			S->Prev = Last;
			S->Next = NULL;
			Last->Next = S;
			Last = S;
		}
	}
	void Delete(T *S)
	{
		if (S->Next==NULL)
		{
			if (S->Prev==NULL)
			{
				First = NULL;
				Last = NULL;
			}
			else
			{
				// last item in the list
				(S->Prev)->Next = NULL;
				Last = (T*)S->Prev;
			}
		}
		else
		{
			if (S->Prev==NULL)
			{
				// first item in the list
				(S->Next)->Prev = NULL;
				First = (T*)S->Next;
			}
			else
			{
				(S->Next)->Prev = S->Prev;
				(S->Prev)->Next = S->Next;
			}
		}
	}
	void Exchange(T *E1, T *E2)
	{
		T *temp;

		if (E1->Next != E2)
		{
			E2->Prev = E1->Prev;
			E1->Prev = E2;
			E1->Next = E2->Next;
			E2->Next = E1;
			return;
		}
		if (E2->Next != E1)
		{
			E1->Prev = E2->Prev;
			E2->Prev = E1;
			E2->Next = E1->Next;
			E1->Next = E2;
			return;
		}

		temp = E1->Prev;
		E1->Prev = E2->Prev;
		E2->Prev = temp;

		temp = E1->Next;
		E1->Next = E2->Next;
		E2->Next = temp;
	}
};


// template class for double-linked lists of scene objects
template <class T> class DListItem;
template <class T> class DList;


template <class T>
class DListItem
{
	friend class DList<T>;
protected:
	DListItem *prev;
	DListItem *next;
	T *me;
public:
	DListItem()  : me(NULL)  {}

	T* Prev()  { return prev->me; }
	T* Next()  { return next->me; }

	DListItem* PrevI()  { return prev; }
	DListItem* NextI()  { return next; }
};


template <class T>
class DList
{
protected:
	DListItem<T> Head;
public:

	DList()  { Head.prev = &Head; Head.next = &Head; }
	~DList()  { assert(Head.me == NULL); }

	T* Last()  { return Head.prev->me; }
	T* First()  { return Head.next->me; }

	DListItem<T>* LastI()  { return Head.prev; }
	DListItem<T>* FirstI()  { return Head.next; }

	void InsertLast(T *S, DListItem<T> *I)
	{
		assert(I->me == NULL);

		I->me = S;

		I->prev = Head.prev;
		I->next = &Head;
		Head.prev->next = I;
		Head.prev = I;
	}

	void Delete(DListItem<T> *I)
	{
		assert(I->me != NULL);

		I->me = NULL;

		I->next->prev = I->prev;
		I->prev->next = I->next;
	}

	void Exchange(DListItem<T> *I1, DListItem<T> *I2)
	{
		assert(I1 != I2);

		DListItem<T> *I1p = I1->prev;
		DListItem<T> *I1n = I1->next;
		DListItem<T> *I2p = I2->prev;
		DListItem<T> *I2n = I2->next;

		I1p->next = I2;
		I1n->prev = I2;
		I2p->next = I1;
		I2n->prev = I1;

		DListItem<T> *temp;

		temp = I1->prev;
		I1->prev = I2->prev;
		I2->prev = temp;

		temp = I1->next;
		I1->next = I2->next;
		I2->next = temp;
	}
};


#endif /* _dlist_h_ */
