/*
 * The POG project
 * Copyright 2000 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */

#include "setup.h"

#ifdef USE_GLUT


#include <stdlib.h>
#include "guiglut.h"
#include "gui.h"
#include "Main.h"
#include "3DMath.h"
#include "Input.h"
#include "Log.h"
#include "InputDevices.h"

#undef GLUT_BUILDING_LIB
#undef GLUT_API_VERSION
#define GLUT_API_VERSION 4
#include <GL/glut.h>



#pragma comment (lib, "glut32.lib")


/* ##################################################
	 # forward declarations of callbacks and stuff
	 # */

static void cbDisplay();
static void cbDisplayNothing();

/* ##################################################
	 # public functions
	 # */

static int guiWindowHandle = 0;


int guiGlutInit()
{
	glutInitWindowSize(guiInfo.winWidth, guiInfo.winHeight);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_RGB | GLUT_DOUBLE);

	glutInit(&guiInfo.argc, guiInfo.argv);

	if (guiInfo.argc <= 1)
	{
		logInfo(modGUI, "Initializing GLUT window to %dx%d.", glutGet(GLUT_INIT_WINDOW_WIDTH), glutGet(GLUT_INIT_WINDOW_HEIGHT));
		if ((guiWindowHandle=glutCreateWindow("VRapps - the POG project")) <= 0)
		{
			logError(modGUI, "Can't create GLUT window.");
			return 0;
		}
	}
	else
	{
		logInfo(modGUI, "Initializing GLUT GameMode screen %s.", guiInfo.argv[1]);
		glutGameModeString(guiInfo.argv[1]);
		if (!glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))
		{
			logError(modGUI, "Can't run GameMode by string \"%s\".", guiInfo.argv[1]);
			return 0;
		}
		if (!glutEnterGameMode())
		{
			logError(modGUI, "Can't enter full screen mode \"%s\".", guiInfo.argv[1]);
			return 0;
		}
		if (!glutGameModeGet(GLUT_GAME_MODE_ACTIVE))
		{
			logError(modGUI, "Error running GameMode by string \"%s\".", guiInfo.argv[1]);
			return 0;
		}
	}

	glutDisplayFunc(cbDisplay);
	glutReshapeFunc(guiWindowReshape);
	glutIdleFunc(guiIdle);

	glutSetCursor(GLUT_CURSOR_NONE);

	return 1;
}


void guiGlutDone()
{
	logInfo(modGUI, "Terminating GLUT window.");

	glutReshapeFunc(NULL);
	glutIdleFunc(NULL);
/*	glutMotionFunc(NULL);
	glutPassiveMotionFunc(NULL);*/
	glutDisplayFunc(cbDisplayNothing);

	if (guiInfo.argc <= 1)
		glutDestroyWindow(guiWindowHandle);
	else
		glutLeaveGameMode();
}


void guiGlutMainLoop()
{
//  logInfo(modGUI, "Entering MainLoop");
	glutMainLoop();
}


void guiTerminateApp()
{
	logInfo(modGUI, "Request for terminating application.");
	guiInfo.Terminated = 1;
	exit(0);
//	DoneApp();
}


void guiForcedTerminate()
{
	guiTerminateApp();
}






/* ##################################################
	 # callbacks
	 # */

void cbDisplay(void)
{
	guiRenderScene();
	glutSwapBuffers();
	guiAfterRendering();
}


void cbDisplayNothing(void)
{
}


void guiIdle(void)
{
	glutPostRedisplay();
}


#endif /* USE_GLUT */

